enum DF_SH_TYPE {
    //% block="软串口（RX=2 TX=3）"
    Software,
    //% block="硬串口（RX=0 TX=1）"
    Hardware
}

enum DF_PLAY_MODE {
    //% block="播放"
    Play,
    //% block="暂停"
    Pause,
    //% block="停止"
    Stop,
    //% block="循环播放"
    Loop,
    //% block="随机播放"
    Random,
    //% block="上一首"
    Previous,
    //% block="下一首"
    Next,
    //% block="音量加"
    VolumeUp,
    //% block="音量减"
    VolumeDown
}

enum SSRXD {
    //% block="0"
    0,
    //% block="1"
    1,
    //% block="2"
    2,
    //% block="3"
    3
}

enum SSTXD {
    //% block="0"
    0,
    //% block="1"
    1,
    //% block="2"
    2,
    //% block="3"
    3
}

//% color="#ffbb00" iconWidth=40 iconHeight=40
namespace DFPlayer {
    //% block="DFPlayer [SH] RX[rx] TX[tx] 初始化" blockType="command"
    //% SH.shadow="dropdown" SH.options="DF_SH_TYPE" SH.defl="DF_SH_TYPE.Software"
    //% rx.shadow="dropdown" rx.options="SSRXD" rx.defl="SSRXD.2" // 默认 RX=2
    //% tx.shadow="dropdown" tx.options="SSTXD" tx.defl="SSTXD.3" // 默认 TX=3
    export function init(parameter: any, block: any) {
        const type_serial = parameter.SH.code;
        let rxpin = parameter.rx.code;
        let txpin = parameter.tx.code;

        // 定义 SSRXD 和 SSTXD 的默认值
        const defaultSoftwareRX = "2"; // 对应 SSRXD.2
        const defaultSoftwareTX = "3"; // 对应 SSTXD.3
        const defaultHardwareRX = "0"; // 对应 SSRXD.0
        const defaultHardwareTX = "1"; // 对应 SSTXD.1

        Generator.addInclude("include_SoftwareSerial", `#include <SoftwareSerial.h>`);
        Generator.addInclude("include_DFPlayer", `#include "dfplayer.h"`);
        Generator.addObject("DFPlayerObject", "DFPlayer", `myDFPlayer;`);

        // 根据串口类型设置默认引脚
        if (type_serial === "Software") {
            // 如果 RX/TX 为硬串口推荐值，强制使用软串口推荐值
            if (rxpin === defaultHardwareRX && txpin === defaultHardwareTX) {
                rxpin = defaultSoftwareRX;
                txpin = defaultSoftwareTX;
            }
            Generator.addSetup("DFPlayer_init", `myDFPlayer.begin(SoftwareSerialPort, ${rxpin}, ${txpin});`);
        } else if (type_serial === "Hardware") {
            // 如果 RX/TX 为软串口推荐值，强制使用硬串口推荐值
            if (rxpin === defaultSoftwareRX && txpin === defaultSoftwareTX) {
                rxpin = defaultHardwareRX;
                txpin = defaultHardwareTX;
            }
            if (Generator.board === 'arduino') {
                Generator.addSetup("DFPlayer_init", `myDFPlayer.begin(HardwareSerialPort);`);
            } else if (Generator.board === 'esp32') {
                Generator.addSetup("DFPlayer_init", `myDFPlayer.begin(HardwareSerialPort, ${rxpin}, ${txpin});`);
            }
        }
    }

    //% block="设置DFPlayer Mini音量为 [VOLUME] %" blockType="command"
    //% VOLUME.shadow="range" VOLUME.params.min=0 VOLUME.params.max=100 VOLUME.defl=50
    export function setVolume(parameter: any, block: any) {
        const vol = parameter.VOLUME.code;
        Generator.addCode(`myDFPlayer.setVolume(${vol});`);
    }

    //% block="设置播放DFPlayer Mini第 [TRACK] 首" blockType="command"
    //% TRACK.shadow="number" TRACK.defl=1
    export function playTrack(parameter: any, block: any) {
        const track = parameter.TRACK.code;
        Generator.addCode(`myDFPlayer.playTrack(${track});`);
    }

    //% block="设置DFPlayer Mini播放模式为[MODE]" blockType="command"
    //% MODE.shadow="dropdown" MODE.options="DF_PLAY_MODE" MODE.defl="DF_PLAY_MODE.Play"
    export function control(parameter: any, block: any) {
        const mode = parameter.MODE.code;
        let code = "";

        switch (mode) {
            case "Play":
                code = "myDFPlayer.playCommand(0x0D);";
                break;
            case "Pause":
                code = "myDFPlayer.playCommand(0x0E);";
                break;
            case "Stop":
                code = "myDFPlayer.playCommand(0x16);";
                break;
            case "Loop":
                code = "myDFPlayer.playCommand(0x11);";
                break;
            case "Random":
                code = "myDFPlayer.playCommand(0x18);";
                break;
            case "Previous":
                code = "myDFPlayer.playCommand(0x02);";
                break;
            case "Next":
                code = "myDFPlayer.playCommand(0x01);";
                break;
            case "VolumeUp":
                code = "myDFPlayer.playCommand(0x04);";
                break;
            case "VolumeDown":
                code = "myDFPlayer.playCommand(0x05);";
                break;
        }

        Generator.addCode(code);
    }
}