#include "dfplayer.h"

#define Start_Byte 0x7E
#define Version_Byte 0xFF
#define Command_Length 0x06
#define End_Byte 0xEF
#define Acknowledge 0x00

DFPlayer::DFPlayer() {
}

DFPlayer::~DFPlayer() {
    if (softSerial != nullptr) {
        delete softSerial;
        softSerial = nullptr;
    }
}

void DFPlayer::begin(DFSerialPort portType, int rxPin, int txPin) {
    if (portType == SoftwareSerialPort) {
        softSerial = new SoftwareSerial(rxPin, txPin);
        softSerial->begin(9600);
        serial = softSerial;
    } else {
        #ifdef ESP32
            Serial1.begin(9600, SERIAL_8N1, rxPin, txPin);
            serial = &Serial1;
        #else
            Serial.begin(9600);
            serial = &Serial;
        #endif
    }
    delay(1000);
}

void DFPlayer::setVolume(uint8_t percent) {
    if (percent > 100) percent = 100;
    uint8_t volume = map(percent, 0, 100, 0, 30);
    sendCommand(0x06, volume);
}

void DFPlayer::playTrack(uint16_t track) {
    sendCommand(0x03, track);
}

void DFPlayer::playCommand(uint8_t cmd) {
    sendCommand(cmd);
}

void DFPlayer::sendCommand(uint8_t cmd, uint16_t param) {
    uint8_t command[10] = {
        Start_Byte,
        Version_Byte,
        Command_Length,
        cmd,
        Acknowledge,
        (uint8_t)(param >> 8),
        (uint8_t)(param & 0xFF),
        0x00,
        0x00,
        End_Byte
    };

    setChecksum(command);
    sendRawCommand(command);
}

void DFPlayer::setChecksum(uint8_t *command) {
    uint16_t sum = 0;
    for (int i = 1; i < 7; i++) {
        sum += command[i];
    }
    sum = 0xFFFF - sum + 1;
    command[7] = (uint8_t)(sum >> 8);
    command[8] = (uint8_t)(sum & 0xFF);
}

void DFPlayer::sendRawCommand(uint8_t *command) {
    for (int i = 0; i < 10; i++) {
        serial->write(command[i]);
    }
}